@@ -38,6 +38,7 @@
 
 static int handle_cmdline(int *argc, char ***argv);
 static int daemonize(void);
+static int open_logs(void);
 static void clean_exit(int sig);
 static void reload_conf(int sig);
 static char *read_line(int fd);
@@ -53,6 +54,7 @@ static const char *socketfile = ACPI_SOCKETFILE;
 static int nosocket;
 static const char *socketgroup;
 static mode_t socketmode = ACPI_SOCKETMODE;
+static int foreground;
 
 int
 main(int argc, char **argv)
@@ -67,12 +69,6 @@ main(int argc, char **argv)
 	/* handle the commandline  */
 	handle_cmdline(&argc, &argv);
 
-	/* check perms */
-	if (geteuid() != 0) {
-		fprintf(stderr, "%s: must be run as root\n", progname);
-		exit(EXIT_FAILURE);
-	}
-
 	/* actually open the event file */
 	event_fd = open(eventfile, O_RDONLY);
 	if (event_fd < 0) {
@@ -81,7 +77,14 @@ main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 	fcntl(event_fd, F_SETFD, FD_CLOEXEC);
-	/* 
+
+/*
+ * if there is data, and the kernel is NOT broken, this eats 1 byte.  We
+ * can't have that.  This is ifdef'ed out on the assumption that old kernels
+ * are out of popular use, by now.
+ */
+#ifdef TEST_FOR_BAD_KERNELS
+	/*
 	 * Older kernels did not support read() properly or poll() at all
 	 * Check that the kernel supports the proper semantics, or die.
 	 *
@@ -106,6 +109,7 @@ main(int argc, char **argv)
 		}
 		fcntl(event_fd, F_SETFL, fl);
 	}
+#endif
 
 	/* open our socket */
 	if (!nosocket) {
@@ -139,11 +143,17 @@ main(int argc, char **argv)
 		}
 	}
 
-	/* if we debug, we don't daemonize */
-	if (!acpid_debug) {
+	/* if we're running in foreground, we don't daemonize */
+	if (!foreground) {
 		if (daemonize() < 0)
 			exit(EXIT_FAILURE);
 	}
+
+	/* normal logging unless debugging */
+	if (!acpid_debug) {
+		if (open_logs() < 0)
+			exit(EXIT_FAILURE);
+	}
 	acpid_log("starting up\n");
 
 	/* trap key signals */
@@ -244,6 +254,7 @@ handle_cmdline(int *argc, char ***argv)
 		{"confdir", 1, 0, 'c'},
 		{"debug", 0, 0, 'd'},
 		{"eventfile", 1, 0, 'e'},
+		{"foreground", 0, 0, 'f'},
 		{"socketgroup", 1, 0, 'g'},
 		{"logfile", 1, 0, 'l'},
 		{"socketmode", 1, 0, 'm'},
@@ -255,8 +266,9 @@ handle_cmdline(int *argc, char ***argv)
 	};
 	const char *opts_help[] = {
 		"Set the configuration directory.",	/* confdir */
-		"Run daemon in the foreground.",	/* debug */
+		"Increase debugging level (implies -f).",/* debug */
 		"Use the specified file for events.",	/* eventfile */
+		"Run in the foreground.",		/* foreground */
 		"Set the group on the socket file.",	/* socketgroup */
 		"Use the specified log file.",		/* logfile */
 		"Set the permissions on the socket file.",/* socketmode */
@@ -271,7 +283,7 @@ handle_cmdline(int *argc, char ***argv)
 
 	for (;;) {
 		int i;
-		i = getopt_long(*argc, *argv, "c:de:g:l:m:s:Svh", opts, NULL);
+		i = getopt_long(*argc, *argv, "c:de:fg:l:m:s:Svh", opts, NULL);
 		if (i == -1) {
 			break;
 		}
@@ -280,11 +292,15 @@ handle_cmdline(int *argc, char ***argv)
 			confdir = optarg;
 			break;
 		case 'd':
+			foreground = 1;
 			acpid_debug++;
 			break;
 		case 'e':
 			eventfile = optarg;
 			break;
+		case 'f':
+			foreground = 1;
+			break;
 		case 'g':
 			socketgroup = optarg;
 			break;
@@ -337,9 +353,6 @@ handle_cmdline(int *argc, char ***argv)
 static int
 daemonize(void)
 {
-	int logfd;
-	int nullfd;
-
 	switch(fork()) {
 	case -1:
 		fprintf(stderr, "%s: fork: %s\n", progname, strerror(errno));
@@ -356,6 +369,18 @@ daemonize(void)
 	setsid();
 	umask(0);
 
+	/* get outta the way */
+	chdir("/");
+
+	return 0;
+}
+
+static int
+open_logs(void)
+{
+	int logfd;
+	int nullfd;
+
 	/* set up stdout, stderr to log and stdin to /dev/null */
 	nullfd = open("/dev/null", O_RDONLY, 0640);
 	if (nullfd < 0) {
@@ -383,9 +408,6 @@ daemonize(void)
 		return -1;
 	}
 
-	/* get outta the way */
-	chdir("/");
-
 	return 0;
 }
 
